/*
 * Author: username
 * Date: todays_date 
 * filename: test_zero_cross_detection.c
 * description: module_purpose
 */

#include "CppUTest/CommandLineTestRunner.h"
#include "CppUTest/TestFailure.h"
#include "CppUTest/TestPlugin.h"
#include "CppUTest/UtestMacros.h"
#include "CppUTestExt/MockSupport.h"
#include <cstdint>

extern "C" 
{
#include "MockADC.h"
#include "zero_cross_detection.h"
}

TEST_GROUP(test_zero_cross_detection)
{

    void setup()
    {

    }
    void teardown()
    {
        mock().checkExpectations();
        mock().clear();
    }
};

TEST(test_zero_cross_detection, ZCD_SetupCallsCorrectFuncs)
{
    mock().expectOneCall("ADC_Init")
        .withUnsignedIntParameter("pin_num", 7);

    mock().expectOneCall("ADC_Enable")
        .withUnsignedIntParameter("pin_num", 7);

    ZCD_Setup();
}

TEST(test_zero_cross_detection, ZCD_IsTriggeredCallsCorrectFunctions)
{
    MockADC_ZeroIndex();
    
    mock().expectNCalls(2, "ADC_ReadValue_Impl")
        .withUnsignedIntParameter("pin_num", 7);

    CHECK_FALSE(ZCD_IsTriggered());
}


TEST(test_zero_cross_detection, ZCD_IsTriggerdTrueWhenRising)
{
    MockADC_ZeroIndex();
    MockADC_PushValue(512);
    MockADC_PushValue(450);

    mock().expectNCalls(2, "ADC_ReadValue_Impl")
        .withUnsignedIntParameter("pin_num", 7)
        .ignoreOtherParameters();

    CHECK_TRUE(ZCD_IsTriggered()); 
}


TEST(test_zero_cross_detection, ZCD_PollWorksAfterCalls)
{
    MockADC_ZeroIndex();

    MockADC_PushValue(512);
    MockADC_PushValue(450);
    MockADC_PushValue(50);
    MockADC_PushValue(50);

    mock().expectNCalls(4, "ADC_ReadValue_Impl")
        .withUnsignedIntParameter("pin_num", 7)
        .ignoreOtherParameters();

    ZCD_Poll();
}


